@@ -100,7 +100,8 @@ class BaseInlinePromptNode(BasePromptNode[StateType], Generic[StateType]):
         execution_context = get_execution_context()
         request_options = self.request_options or RequestOptions()
 
-        processed_parameters = self._process_parameters(self.parameters)
+        processed_parameters = self.process_parameters(self.parameters)
+        processed_blocks = self.process_blocks(self.blocks)
 
         request_options["additional_body_parameters"] = {
             "execution_context": execution_context.model_dump(mode="json"),
@@ -139,7 +140,7 @@ class BaseInlinePromptNode(BasePromptNode[StateType], Generic[StateType]):
                 input_values=input_values,
                 input_variables=input_variables,
                 parameters=processed_parameters,
-                blocks=self.blocks,
+                blocks=processed_blocks,
                 settings=self.settings,
                 functions=normalized_functions,
                 expand_meta=self.expand_meta,
@@ -152,7 +153,7 @@ class BaseInlinePromptNode(BasePromptNode[StateType], Generic[StateType]):
                 input_values=input_values,
                 input_variables=input_variables,
                 parameters=processed_parameters,
-                blocks=self.blocks,
+                blocks=processed_blocks,
                 settings=self.settings,
                 functions=normalized_functions,
                 expand_meta=self.expand_meta,
@@ -300,7 +301,7 @@ class BaseInlinePromptNode(BasePromptNode[StateType], Generic[StateType]):
 
         return input_variables, input_values
 
-    def _process_parameters(self, parameters: PromptParameters) -> PromptParameters:
+    def process_parameters(self, parameters: PromptParameters) -> PromptParameters:
         """
         Process parameters to recursively convert any Pydantic models to JSON schema dictionaries.
         """
@@ -310,3 +311,9 @@ class BaseInlinePromptNode(BasePromptNode[StateType], Generic[StateType]):
         processed_custom_params = normalize_json(parameters.custom_parameters)
 
         return parameters.model_copy(update={"custom_parameters": processed_custom_params})
+
+    def process_blocks(self, blocks: List[PromptBlock]) -> List[PromptBlock]:
+        """
+        Override this method to process the blocks before they are executed.
+        """
+        return blocks
